@@ -3,11 +3,11 @@
 
 use crate::{agent::agent_error::HttpErrorPayload, ic_types::Principal, AgentError, RequestId};
 use reqwest::Method;
-use std::{future::Future, pin::Pin};
+use std::{future::Future, pin::Pin, sync::Arc};
 
 /// Implemented by the Agent environment to cache and update an HTTP Auth password.
 /// It returns a tuple of `(username, password)`.
-pub trait PasswordManager {
+pub trait PasswordManager: Send + Sync {
     /// Retrieve the cached value for a user. If no cache value exists for this URL,
     /// the manager can return [`None`].
     fn cached(&self, url: &str) -> Result<Option<(String, String)>, String>;
@@ -21,14 +21,56 @@ pub trait PasswordManager {
     fn required(&self, url: &str) -> Result<(String, String), String>;
 }
 
-/// A [ReplicaV2Transport] using Reqwest to make HTTP calls to the internet computer.
-pub struct ReqwestHttpReplicaV2Transport {
-    url: reqwest::Url,
-    client: reqwest::Client,
-    password_manager: Option<Box<dyn PasswordManager + Send + Sync>>,
+/// A type which can never be created used indicate no password manager.
+///
+/// This type is a stable equivalent to the `!` type from `std`.
+///
+/// This is currently an alias for [`std::convert::Infallible`], but in
+/// the future it may be an alias for [`!`][never].
+/// See ["Future compatibility" section of `std::convert::Infallible`][infallible] for more.
+///
+/// [never]: https://doc.rust-lang.org/nightly/std/primitive.never.html
+/// [infallible]: https://doc.rust-lang.org/nightly/std/convert/enum.Infallible.html#future-compatibility
+pub type NoPasswordManager = std::convert::Infallible;
+
+impl<P: PasswordManager + ?Sized> PasswordManager for Box<P> {
+    fn cached(&self, url: &str) -> Result<Option<(String, String)>, String> {
+        (**self).cached(url)
+    }
+    fn required(&self, url: &str) -> Result<(String, String), String> {
+        (**self).required(url)
+    }
+}
+impl<P: PasswordManager + ?Sized> PasswordManager for Arc<P> {
+    fn cached(&self, url: &str) -> Result<Option<(String, String)>, String> {
+        (**self).cached(url)
+    }
+    fn required(&self, url: &str) -> Result<(String, String), String> {
+        (**self).required(url)
+    }
 }
 
-impl ReqwestHttpReplicaV2Transport {
+impl PasswordManager for NoPasswordManager {
+    fn cached(&self, _: &str) -> Result<Option<(String, String)>, String> {
+        unreachable!()
+    }
+    fn required(&self, _: &str) -> Result<(String, String), String> {
+        unreachable!()
+    }
+}
+
+/// A [ReplicaV2Transport] using Reqwest to make HTTP calls to the internet computer.
+pub type ReqwestHttpReplicaV2Transport =
+    ReqwestHttpReplicaV2TransportImpl<Box<dyn PasswordManager>>;
+
+/// A [ReplicaV2Transport] using Reqwest to make HTTP calls to the internet computer.
+pub struct ReqwestHttpReplicaV2TransportImpl<P: PasswordManager> {
+    url: reqwest::Url,
+    client: reqwest::Client,
+    password_manager: Option<P>,
+}
+
+impl<P: PasswordManager> ReqwestHttpReplicaV2TransportImpl<P> {
     pub fn create<U: Into<String>>(url: U) -> Result<Self, AgentError> {
         let mut tls_config = rustls::ClientConfig::new();
 
@@ -53,16 +95,25 @@ impl ReqwestHttpReplicaV2Transport {
         })
     }
 
-    pub fn with_password_manager<P: 'static + PasswordManager + Send + Sync>(
+    pub fn with_password_manager<P1: PasswordManager>(
         self,
-        password_manager: P,
-    ) -> Self {
-        Self {
-            password_manager: Some(Box::new(password_manager)),
-            ..self
+        password_manager: P1,
+    ) -> ReqwestHttpReplicaV2TransportImpl<P1> {
+        ReqwestHttpReplicaV2TransportImpl {
+            password_manager: Some(password_manager),
+            url: self.url,
+            client: self.client,
         }
     }
 
+    pub fn password_manager(&self) -> &Option<P> {
+        &self.password_manager
+    }
+
+    pub fn password_manager_mut(&mut self) -> &mut Option<P> {
+        &mut self.password_manager
+    }
+
     fn maybe_add_authorization(
         &self,
         http_request: &mut reqwest::Request,
@@ -166,15 +217,15 @@ impl ReqwestHttpReplicaV2Transport {
     }
 }
 
-impl super::ReplicaV2Transport for ReqwestHttpReplicaV2Transport {
+impl<P: PasswordManager> super::ReplicaV2Transport for ReqwestHttpReplicaV2TransportImpl<P> {
     fn call<'a>(
         &'a self,
         effective_canister_id: Principal,
         envelope: Vec<u8>,
         _request_id: RequestId,
     ) -> Pin<Box<dyn Future<Output = Result<(), AgentError>> + Send + 'a>> {
-        async fn run(
-            s: &ReqwestHttpReplicaV2Transport,
+        async fn run<P: PasswordManager>(
+            s: &ReqwestHttpReplicaV2TransportImpl<P>,
             effective_canister_id: Principal,
             envelope: Vec<u8>,
         ) -> Result<(), AgentError> {
@@ -191,8 +242,8 @@ impl super::ReplicaV2Transport for ReqwestHttpReplicaV2Transport {
         effective_canister_id: Principal,
         envelope: Vec<u8>,
     ) -> Pin<Box<dyn Future<Output = Result<Vec<u8>, AgentError>> + Send + 'a>> {
-        async fn run(
-            s: &ReqwestHttpReplicaV2Transport,
+        async fn run<P: PasswordManager>(
+            s: &ReqwestHttpReplicaV2TransportImpl<P>,
             effective_canister_id: Principal,
             envelope: Vec<u8>,
         ) -> Result<Vec<u8>, AgentError> {
@@ -208,8 +259,8 @@ impl super::ReplicaV2Transport for ReqwestHttpReplicaV2Transport {
         effective_canister_id: Principal,
         envelope: Vec<u8>,
     ) -> Pin<Box<dyn Future<Output = Result<Vec<u8>, AgentError>> + Send + 'a>> {
-        async fn run(
-            s: &ReqwestHttpReplicaV2Transport,
+        async fn run<P: PasswordManager>(
+            s: &ReqwestHttpReplicaV2TransportImpl<P>,
             effective_canister_id: Principal,
             envelope: Vec<u8>,
         ) -> Result<Vec<u8>, AgentError> {
@@ -223,7 +274,9 @@ impl super::ReplicaV2Transport for ReqwestHttpReplicaV2Transport {
     fn status<'a>(
         &'a self,
     ) -> Pin<Box<dyn Future<Output = Result<Vec<u8>, AgentError>> + Send + 'a>> {
-        async fn run(s: &ReqwestHttpReplicaV2Transport) -> Result<Vec<u8>, AgentError> {
+        async fn run<P: PasswordManager>(
+            s: &ReqwestHttpReplicaV2TransportImpl<P>,
+        ) -> Result<Vec<u8>, AgentError> {
             s.execute(Method::GET, "status", None).await
         }
 
